@@ -10,11 +10,12 @@ gi.require_version('Gdk', '4.0')
 gi.require_version('Gtk', '4.0')
 gi.require_version('Adw', '1')
 gi.require_version('WebKit', '6.0')
-from gi.repository import GLib, Gio, Gdk, Gtk, Adw
+from gi.repository import GLib, Gio, Gdk, Gtk, Adw, WebKit
 
 from wike.data import settings, languages, history, bookmarks
 from wike.prefs import PrefsWindow
 from wike.window import Window
+from wike.view import network_session
 
 
 # Main application class for Wike
@@ -43,15 +44,17 @@ class Application(Adw.Application):
     Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(), css_widgets, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
     self._style_manager = Adw.StyleManager.get_default()
-    if settings.get_int('theme') == 0:
-      self._style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)
-    elif settings.get_int('theme') == 1:
-      self._style_manager.set_color_scheme(Adw.ColorScheme.FORCE_DARK)
-    elif settings.get_int('theme') == 2:
-      self._style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)
-      Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(), self._css_sepia, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
-    elif settings.get_int('theme') == 3:
-      self._style_manager.set_color_scheme(Adw.ColorScheme.PREFER_LIGHT)
+    theme = settings.get_int('theme')
+    match theme:
+      case 0:
+        self._style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)
+      case 1:
+        self._style_manager.set_color_scheme(Adw.ColorScheme.FORCE_DARK)
+      case 2:
+        self._style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)
+        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(), self._css_sepia, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
+      case 3:
+        self._style_manager.set_color_scheme(Adw.ColorScheme.PREFER_LIGHT)
 
     action = Gio.SimpleAction.new('prefs', None)
     action.connect('activate', self._prefs_cb)
@@ -184,10 +187,29 @@ class Application(Adw.Application):
     else:
       settings.set_string('last-uri', self._window.page.wikiview.get_base_uri())
 
-    settings.sync()
-    languages.save()
+    tabs_data = {}
+    for i_page in range(self._window.tabview.get_n_pages()):
+      tabpage = self._window.tabview.get_nth_page(i_page)
+      page = tabpage.get_child()
+      if page.lazy_load:
+        uri = page.lazy_load[0]
+        title = page.lazy_load[1]
+      else:
+        uri = page.wikiview.get_base_uri()
+        title = page.wikiview.title
+      if not uri.startswith('about'):
+        tabs_data[uri] = title
+    settings.set_value('last-tabs', GLib.Variant('a{ss}', tabs_data))
+
     if not settings.get_boolean('keep-history'):
       history.clear()
+
+    if settings.get_boolean('clear-data'):
+      data_manager = network_session.get_website_data_manager()
+      data_manager.clear(WebKit.WebsiteDataTypes.ALL, 0, None, None, None)
+
+    settings.sync()
+    languages.save()
     history.save()
     bookmarks.save()
 
