@@ -86,6 +86,22 @@ class History:
     else:
       self.items[date] = {uri: [time, title, lang]}
 
+  # Remove item from history list
+
+  def remove(self, date, uri):
+    item_deleted = False
+    date_deleted = False
+
+    if date in self.items:
+      if uri in self.items[date]:
+        del self.items[date][uri]
+        item_deleted = True
+      if len(self.items[date]) == 0:
+        del self.items[date]
+        date_deleted = True
+
+    return item_deleted, date_deleted
+
   # Clear history
 
   def clear(self):
