@@ -20,7 +20,6 @@ class SearchBox(Gtk.Box):
   __gtype_name__ = 'SearchBox'
   
   search_entry = Gtk.Template.Child()
-  lang_label = Gtk.Template.Child()
   settings_button = Gtk.Template.Child()
   
   # Create suggestions list and connect search entry events
@@ -52,9 +51,9 @@ class SearchBox(Gtk.Box):
 
   # Search text in Wikipedia and load results list
 
-  def _search_wikipedia(self, text):
+  def _search_wikipedia(self, text, lang):
     try:
-      self.results_list = wikipedia.search(text.lower(), settings.get_string('search-language'), 10)
+      self.results_list = wikipedia.search(text, lang, 10)
     except:
       self.results_list = None
     self._results_changed = True
@@ -81,14 +80,30 @@ class SearchBox(Gtk.Box):
   def _search_changed_cb(self, search_entry):
     search_entry.grab_focus()
     if settings.get_boolean('search-suggestions'):
-      text = search_entry.get_text()
+      text, lang = self._get_search_terms(search_entry.get_text())
       if len(text) > 2:
-        t = Thread(target=self._search_wikipedia, args=(text, ))
+        t = Thread(target=self._search_wikipedia, args=(text, lang, ))
         t.start()
       else:
         self.results_list = None
         self._results_changed = True
 
+  # Split search terms to get language prefix
+
+  def _get_search_terms(self, text):
+    text = text.lower()
+    lang = settings.get_string('search-language')
+    if text.startswith('-'):
+      terms = text.split(' ', 1)
+      prefix = terms[0].lstrip('-')
+      if prefix in languages.wikilangs.keys():
+        lang = prefix
+      if len(terms) > 1:
+        text = terms[1]
+      else:
+        text = ''
+    return text, lang
+
   # When search stoped with ESC hide suggestions
   
   def _search_stop_cb(self, search_entry):
@@ -119,12 +134,12 @@ class SearchBox(Gtk.Box):
   # On entry activated search for text in Wikipedia and load result
 
   def _search_activate_cb(self, search_entry):
-    text = search_entry.get_text()
+    text, lang = self._get_search_terms(search_entry.get_text())
     self.suggestions_popover.hide()
     search_entry.delete_text(0, -1)
     if text != '':
       try:
-        result = wikipedia.search(text.lower(), settings.get_string('search-language'), 1)
+        result = wikipedia.search(text, lang, 1)
       except:
         self.window.page.wikiview.load_message('notfound')
       else:
@@ -196,7 +211,6 @@ class SettingsPopover(Gtk.Popover):
 
   __gtype_name__ = 'SettingsPopover'
 
-  filter_entry = Gtk.Template.Child()
   languages_button = Gtk.Template.Child()
   languages_list = Gtk.Template.Child()
   languages_scroller = Gtk.Template.Child()
@@ -211,36 +225,22 @@ class SettingsPopover(Gtk.Popover):
     self._search_box = search_box
     self._window = search_box.window
 
-    self.languages_list.set_filter_func(self._filter_list)
     self.populate_list()
 
     settings.bind('search-suggestions', self.suggestions_switch, 'active', Gio.SettingsBindFlags.DEFAULT)
     settings.connect('changed::show-flags', self._settings_show_flags_changed_cb)
     
     self.languages_button.connect('clicked', self._languages_button_clicked_cb)
-    self.filter_entry.connect('search-changed', self._filter_entry_changed_cb)
     self.languages_list.connect('row-activated', self._languages_list_activated_cb)
     self.languages_scroller.get_vadjustment().connect('value-changed', self._languages_scrolled_cb)
     self.connect('show', self._popover_show_cb)
 
-  # Filter languages list for filter entry content
-
-  def _filter_list(self, row):
-    text = self.filter_entry.get_text()
-    if text == '':
-      return True
-
-    if row.lang_name.lower().startswith(text.lower()) or row.lang_id.lower().startswith(text.lower()):
-      return True
-    else:
-      return False
-
   # Populate languages list
 
   def populate_list(self):
     lang_id = settings.get_string('search-language')
-    self._search_box.lang_label.set_label(lang_id)
-    self._search_box.lang_label.set_tooltip_text(languages.wikilangs[lang_id].capitalize())
+    self._search_box.settings_button.set_label(lang_id)
+    self._search_box.settings_button.set_tooltip_text(languages.wikilangs[lang_id].capitalize())
 
     while True:
       row = self.languages_list.get_row_at_index(0)
@@ -269,11 +269,6 @@ class SettingsPopover(Gtk.Popover):
     languages_window.set_transient_for(self._window)
     languages_window.show()
 
-  # On filter entry changed refresh languages list
-
-  def _filter_entry_changed_cb(self, filter_entry):
-    self.languages_list.invalidate_filter()
-
   # On list activated change search language
 
   def _languages_list_activated_cb(self, lang_list, row):
