@@ -5,7 +5,7 @@
 
 import os
 
-from gi.repository import GLib, Gio, Gdk, Gtk, Adw, WebKit
+from gi.repository import GLib, GObject, Gio, Gdk, Gtk, Adw, WebKit
 
 from wike.data import settings
 from wike.bookmarks import BookmarksBox
@@ -27,6 +27,7 @@ class Window(Adw.ApplicationWindow):
   toast_overlay = Gtk.Template.Child()
   flap = Gtk.Template.Child()
   flap_stack = Gtk.Template.Child()
+  flap_pin_revealer = Gtk.Template.Child()
   flap_pin_button = Gtk.Template.Child()
   flap_toc_button = Gtk.Template.Child()
   flap_langlinks_button = Gtk.Template.Child()
@@ -53,8 +54,7 @@ class Window(Adw.ApplicationWindow):
     if settings.get_boolean('window-max'):
       self.maximize()
 
-    self.page = PageBox(self)
-    tabpage = self.tabview.append(self.page)
+    self._print_settings = Gtk.PrintSettings()
 
     self.actionbar = ActionBar()
     self.window_box.append(self.actionbar)
@@ -74,16 +74,58 @@ class Window(Adw.ApplicationWindow):
     self.history_box = HistoryBox(self)
     history_stack_page = self.flap_stack.add_named(self.history_box, 'history')
 
-    self._print_settings = Gtk.PrintSettings()
+    flap_page = settings.get_string('flap-page')
+    match flap_page:
+      case 'langlinks':
+        self.flap_langlinks_button.set_active(True)
+      case 'bookmarks':
+        self.flap_bookmarks_button.set_active(True)
+      case 'history':
+        self.flap_history_button.set_active(True)
+      case _:
+        self.flap_toc_button.set_active(True)
 
-    if settings.get_string('flap-page') == 'langlinks':
-      self.flap_langlinks_button.set_active(True)
-    elif settings.get_string('flap-page') == 'bookmarks':
-      self.flap_bookmarks_button.set_active(True)
-    elif settings.get_string('flap-page') == 'history':
-      self.flap_history_button.set_active(True)
+    self.page = PageBox(self, None)
+
+    if launch_uri != '':
+      tabpage = self.tabview.append(self.page)
+      if launch_uri == 'notfound':
+        self.page.wikiview.load_message(launch_uri)
+      else:
+        self.page.wikiview.load_wiki(launch_uri)
     else:
-      self.flap_toc_button.set_active(True)
+      on_start = settings.get_int('on-start-load')
+      last_uri = settings.get_string('last-uri')
+      if on_start != 3:
+        tabpage = self.tabview.append(self.page)
+        match on_start:
+          case 0:
+            self.page.wikiview.load_main()
+          case 1:
+            self.page.wikiview.load_random()
+          case 2:
+            if last_uri != '':
+              self.page.wikiview.load_wiki(last_uri)
+            else:
+              self.page.wikiview.load_message('blank')
+      else:
+        tabs_data = settings.get_value('last-tabs').unpack()
+        if len(tabs_data) > 0:
+          for uri, title in tabs_data.items():
+            if last_uri == uri:
+              tabpage = self.tabview.append(self.page)
+              self.tabview.set_selected_page(tabpage)
+              self.page.wikiview.load_wiki(last_uri)
+            else:
+              self.new_lazy_page(uri, title, None)
+          if not last_uri in tabs_data:
+            tabpage = self.tabview.get_nth_page(0)
+            self.tabview.set_selected_page(tabpage)
+            self.page = tabpage.get_child()
+            self.page.load_now()
+        else:
+          tabpage = self.tabview.append(self.page)
+          self.page.wikiview.load_message('blank')
 
     settings.bind('flap-pinned', self.flap_pin_button, 'active', Gio.SettingsBindFlags.DEFAULT)
     settings.bind('flap-page', self.flap_stack, 'visible-child-name', Gio.SettingsBindFlags.DEFAULT)
@@ -100,21 +142,9 @@ class Window(Adw.ApplicationWindow):
     self.flap_bookmarks_button.connect('toggled', self._flap_switcher_button_cb, 'bookmarks')
     self.flap_history_button.connect('toggled', self._flap_switcher_button_cb, 'history')
 
-    if launch_uri == 'notfound':
-      self.page.wikiview.load_message(launch_uri)
-    else:
-      if launch_uri != '':
-        self.page.wikiview.load_wiki(launch_uri)
-      else:
-        if settings.get_int('on-start-load') == 0:
-          self.page.wikiview.load_main()
-        elif settings.get_int('on-start-load') == 1:
-          self.page.wikiview.load_random()
-        else:
-          if settings.get_string('last-uri'):
-            self.page.wikiview.load_wiki(settings.get_string('last-uri'))
-          else:
-            self.page.wikiview.load_main()
+    gesture = Gtk.GestureClick(button=0, propagation_phase=Gtk.PropagationPhase.CAPTURE)
+    self.add_controller(gesture)
+    gesture.connect('pressed', self._gesture_click_cb)
 
   # Set actions for window
   
@@ -125,6 +155,7 @@ class Window(Adw.ApplicationWindow):
                 ('close-tab', self._close_tab_cb, ('<Ctrl>W',)),
                 ('next-tab', self._next_tab_cb, ('<Ctrl>Tab',)),
                 ('prev-tab', self._prev_tab_cb, ('<Shift><Ctrl>Tab',)),
+                ('toggle-overview', self._toggle_overview_cb, ('F4', '<Alt>T',)),
                 ('go-search', self._go_search_cb, ('F2', '<Ctrl>K',)),
                 ('add-bookmark', self._add_bookmark_cb, ('<Ctrl>D',)),
                 ('show-toc', self._show_toc_cb, ('<Ctrl>I',)),
@@ -153,6 +184,7 @@ class Window(Adw.ApplicationWindow):
 
     pin_sidebar_action = Gio.SimpleAction.new_stateful('pin-sidebar', None, GLib.Variant.new_boolean(False))
     pin_sidebar_action.connect('change-state', self._pin_sidebar_cb)
+    pin_sidebar_action.bind_property('enabled', self.flap_pin_revealer, 'reveal-child',  GObject.BindingFlags.SYNC_CREATE)
     self.add_action(pin_sidebar_action)
     
     self.flap.connect('notify::reveal-flap', self._flap_reveal_cb)
@@ -160,6 +192,21 @@ class Window(Adw.ApplicationWindow):
     if settings.get_boolean('flap-pinned'):
       pin_sidebar_action.change_state(GLib.Variant.new_boolean(True))
 
+  # Handle mouse clicks with buttons 8 and 9
+
+  def _gesture_click_cb(self, gesture, n_press, x, y):
+    button = gesture.get_current_button()
+    prev_page_action = self.lookup_action('prev-page')
+    next_page_action = self.lookup_action('next-page')
+
+    match button:
+      case 8:
+        gesture.set_state(Gtk.EventSequenceState.CLAIMED)
+        prev_page_action.activate()
+      case 9:
+        gesture.set_state(Gtk.EventSequenceState.CLAIMED)
+        next_page_action.activate()
+
   # If window size changed set layout
 
   def do_size_allocate(self, width, height, baseline):
@@ -179,29 +226,43 @@ class Window(Adw.ApplicationWindow):
   def _set_layout (self):
     pin_sidebar_action = self.lookup_action('pin-sidebar')
 
-    if self.mobile_layout:
-      self.headerbar.set_mobile(True)
-      pin_sidebar_action.change_state(GLib.Variant.new_boolean(False))
-      pin_sidebar_action.set_enabled(False)
-    else:
-      self.headerbar.set_mobile(False)
-      pin_sidebar_action.set_enabled(True)
+    self.headerbar.set_mobile(self.mobile_layout)
+    pin_sidebar_action.set_enabled(not self.mobile_layout)
+
+    self._update_sidebar_policy(pin_sidebar_action)
 
   # Create new tab with a page
 
   def new_page(self, uri, parent, select):
-    page = PageBox(self)
+    page = PageBox(self, None)
     tabpage = self.tabview.add_page(page, parent)
-    if uri == 'blank' or uri == 'notfound':
-      page.wikiview.load_message(uri)
-    else:
-      page.wikiview.load_wiki(uri)
+    tabpage.set_live_thumbnail(True)
+
+    match uri:
+      case 'main':
+        page.wikiview.load_main()
+      case 'random':
+        page.wikiview.load_random()
+      case 'blank' | 'notfound':
+        page.wikiview.load_message(uri)
+      case _:
+        page.wikiview.load_wiki(uri)
 
     if select:
       self.tabview.set_selected_page(tabpage)
 
     return tabpage
 
+  # New empty tab with a title for lazy-loading
+
+  def new_lazy_page(self, uri, title, parent):
+    page = PageBox(self, [uri,title])
+    tabpage = self.tabview.add_page(page, parent)
+    tabpage.set_live_thumbnail(True)
+    tabpage.set_title(title)
+
+    return tabpage
+
   # New empty tab
 
   def _new_tab_cb(self, action, parameter):
@@ -242,7 +303,12 @@ class Window(Adw.ApplicationWindow):
       tabpage = self.tabview.get_nth_page(num_pages - 1)
       self.tabview.set_selected_page(tabpage)
 
-  # On tab selected event refresh headerbar and sidebar
+  # Open/close taboverview
+
+  def _toggle_overview_cb(self, action, parameter):
+    self.taboverview.set_open(not self.taboverview.get_open())
+
+  # On tab selected event refresh headerbar and sidebar, and force loading
 
   def _tabview_selected_page_cb(self, tabview, value):
     tabpage = tabview.get_selected_page()
@@ -250,11 +316,14 @@ class Window(Adw.ApplicationWindow):
       return
 
     self.page = tabpage.get_child()
-    self.refresh_nav_actions(self.page.wikiview)
-    self.refresh_menu_actions(self.page.wikiview.is_local())
-    self.toc_box.populate(self.page.wikiview.title, self.page.wikiview.sections)
-    self.langlinks_box.populate(self.page.wikiview.langlinks)
-    self.bookmarks_box.refresh_buttons()
+    if self.page.lazy_load:
+      self.page.load_now()
+    else:
+      self.refresh_nav_actions(self.page.wikiview)
+      self.refresh_menu_actions(self.page.wikiview.is_local())
+      self.toc_box.populate(self.page.wikiview.title, self.page.wikiview.sections)
+      self.langlinks_box.populate(self.page.wikiview.langlinks)
+      self.bookmarks_box.refresh_buttons()
 
   # On tab closed event destroy wikiview and confirm
 
@@ -322,12 +391,17 @@ class Window(Adw.ApplicationWindow):
     else:
       self.flap.set_reveal_flap(False)
   
-  # Pin sidebar (flap)
+  # On pin sidebar action set state and update sidebar
   
   def _pin_sidebar_cb(self, action, parameter):
     action.set_state(parameter)
-    
-    if parameter:
+    self._update_sidebar_policy(action)
+
+  # Update sidebar policy
+
+  def _update_sidebar_policy(self, action):
+    prefer_pinned = action.get_state().get_boolean()
+    if prefer_pinned and not self.mobile_layout:
       self.flap.set_fold_policy(Adw.FlapFoldPolicy.NEVER)
     else:
       self.flap.set_fold_policy(Adw.FlapFoldPolicy.ALWAYS)
